package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.IOrganizeService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.Organize;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description : 组织对外接口
 * @Reference   :
 * @Author      : wei.peng
 * @Date        : 2018-10-22 16:58:42.972
 * @Modify      :
 **/
@RestController
@Api(description = "组织")
@RequestMapping(CoreBaseController.BASE_URL +"/organize")
public class OrganizeController extends CoreBaseController{

    public static final Logger LOGGER = LoggerFactory.getLogger(OrganizeController.class);

    @Autowired
    private IOrganizeService organizeService;

    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增组织", notes = "组织")
    public ResultBean insertOrganize(Organize organize) {
        try {
            //条件验证
            ValidatorBean.beginValid(organize)
                    .notNull("name", organize.getName())
                    .notNull("organizeCode", organize.getOrganizeCode())
                    .notNull("organizeTypeId", organize.getOrganizeTypeId());

            //新增初始化
            ConvertBean.modelInitialize(organize, AuthUtil.getSessionUser());
            if(organize.getParentId() == null || organize.getParentId().intValue() == 0){
                organize.setParentId(CommonEnumUtil.PARENT.DEFAULT.getValue());
            }

            organize = organizeService.insertOrganize(organize);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(organize);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep).build();
        } catch (Exception e) {
            return ResultBean.fail(e.getMessage()).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @PutMapping(value = "/update")
    @ApiOperation(value = "修改组织", notes = "组织")
    public ResultBean updateOrganize(Organize organize) {
        try {
            // 数据校验
            ValidatorBean.beginValid(organize)
                    .notNull("id", organize.getId())
                    .notNull("name", organize.getName())
                    .notNull("organizeCode", organize.getOrganizeCode())
                    .notNull("organizeTypeId", organize.getOrganizeTypeId());

            ConvertBean.modelUpdate(organize, AuthUtil.getSessionUser());
            if(organize.getParentId() == null || organize.getParentId().intValue() == 0){
                organize.setParentId(CommonEnumUtil.PARENT.DEFAULT.getValue());
            }

            organizeService.updateOrganize(organize);
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow()).build();
        } catch (Exception e) {
            return ResultBean.fail(e.getMessage()).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation(value = "删除组织", notes = "删除组织")
    public ResultBean deleteOrganize(@PathVariable("id") String idStr) {
        try {
            // 数据校验
            ValidatorBean.checkIsNumber(idStr.toString(), "您要删除的组织不存在");

            organizeService.deleteOrganizeById(idStr);
            return ResultBean.success().setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/list")
    @ApiOperation(value = "查询Organize", notes = "查询所有组织")
    public ResultBean findOrganizeAll() {
        try {
            List<Organize> list = organizeService.listOrganize();
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value="/query")
    @ApiOperation(value="查询组织",notes="组合查询组织外加分页")
    public ResultBean queryPosition(Organize organize, Pager pager){
        try{
            ListPager<Organize> listPager = organizeService.queryOrganize(organize, pager);
            return ResultBean.success("操作成功")
                    .setListPager(listPager)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/get/{id}")
    @ApiOperation(value = "查询组织", notes = "查询根据ID查询组织")
    public ResultBean getByOrganizeId(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "角色id 不能为空");

            Organize organize = organizeService.getOrganizeById(id);

            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(organize);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            e.printStackTrace();
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @PutMapping(value = "/status")
    @ApiOperation(value = "修改组织状态", notes = "根据ID修改组织状态")
    public ResultBean updateStatus(String id, int status) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "组织id 不能为空");
            ValidatorBean.checkNotZero(status, "组织状态不能为空");

            organizeService.updateOrganizeStatus(id, status, AuthUtil.getSessionUser());
            return new ResultBean(true);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode());
        }
    }

    @DeleteMapping(value = "/batch-delete")
    @ApiOperation(value = "根据id批量删除", notes = "根据id批量删除")
    public ResultBean deleteOrganizeByIds(String[] ids){
        try{
            ids = ConvertBean.modelSafeArrayNumber(ids,true);
            if(ids.length == 0){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的资源。")
                        .build();
            }

            organizeService.deleteOrganizeByIds(ids);
            return ResultBean.success("删除成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }
}
